# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.order_tracker_response import OrderTrackerResponse
from paypalserversdk.models.phone_number_with_country_code import PhoneNumberWithCountryCode
from paypalserversdk.models.shipping_name import ShippingName
from paypalserversdk.models.shipping_option import ShippingOption


class ShippingWithTrackingDetails(object):

    """Implementation of the 'Shipping With Tracking Details' model.

    The order shipping details.

    Attributes:
        name (ShippingName): The name of the party.
        phone_number (PhoneNumberWithCountryCode): The phone number in its
            canonical international [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en).
        mtype (FullfillmentType): A classification for the method of purchase
            fulfillment (e.g shipping, in-store pickup, etc). Either `type` or
            `options` may be present, but not both.
        options (List[ShippingOption]): An array of shipping options that the
            payee or merchant offers to the payer to ship or pick up their
            items.
        address (Address): The portable international postal address. Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).
        trackers (List[OrderTrackerResponse]): An array of trackers for a
            transaction.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "phone_number": 'phone_number',
        "mtype": 'type',
        "options": 'options',
        "address": 'address',
        "trackers": 'trackers'
    }

    _optionals = [
        'name',
        'phone_number',
        'mtype',
        'options',
        'address',
        'trackers',
    ]

    def __init__(self,
                 name=APIHelper.SKIP,
                 phone_number=APIHelper.SKIP,
                 mtype=APIHelper.SKIP,
                 options=APIHelper.SKIP,
                 address=APIHelper.SKIP,
                 trackers=APIHelper.SKIP):
        """Constructor for the ShippingWithTrackingDetails class"""

        # Initialize members of the class
        if name is not APIHelper.SKIP:
            self.name = name 
        if phone_number is not APIHelper.SKIP:
            self.phone_number = phone_number 
        if mtype is not APIHelper.SKIP:
            self.mtype = mtype 
        if options is not APIHelper.SKIP:
            self.options = options 
        if address is not APIHelper.SKIP:
            self.address = address 
        if trackers is not APIHelper.SKIP:
            self.trackers = trackers 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = ShippingName.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        phone_number = PhoneNumberWithCountryCode.from_dictionary(dictionary.get('phone_number')) if 'phone_number' in dictionary.keys() else APIHelper.SKIP
        mtype = dictionary.get("type") if dictionary.get("type") else APIHelper.SKIP
        options = None
        if dictionary.get('options') is not None:
            options = [ShippingOption.from_dictionary(x) for x in dictionary.get('options')]
        else:
            options = APIHelper.SKIP
        address = Address.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        trackers = None
        if dictionary.get('trackers') is not None:
            trackers = [OrderTrackerResponse.from_dictionary(x) for x in dictionary.get('trackers')]
        else:
            trackers = APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   phone_number,
                   mtype,
                   options,
                   address,
                   trackers)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class SellerProtectionStatus(object):

    """Implementation of the 'Seller Protection Status' enum.

    Indicates whether the transaction is eligible for seller protection. For
    information, see [PayPal Seller Protection for
    Merchants](https://www.paypal.com/us/webapps/mpp/security/seller-protection
    ).

    Attributes:
        ELIGIBLE: TODO: type description here.
        PARTIALLY_ELIGIBLE: TODO: type description here.
        NOT_ELIGIBLE: TODO: type description here.

    """
    ELIGIBLE = 'ELIGIBLE'

    PARTIALLY_ELIGIBLE = 'PARTIALLY_ELIGIBLE'

    NOT_ELIGIBLE = 'NOT_ELIGIBLE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CardType(object):

    """Implementation of the 'Card Type' enum.

    Type of card. i.e Credit, Debit and so on.

    Attributes:
        CREDIT: TODO: type description here.
        DEBIT: TODO: type description here.
        PREPAID: TODO: type description here.
        STORE: TODO: type description here.
        UNKNOWN: TODO: type description here.

    """
    CREDIT = 'CREDIT'

    DEBIT = 'DEBIT'

    PREPAID = 'PREPAID'

    STORE = 'STORE'

    UNKNOWN = 'UNKNOWN'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PhoneType(object):

    """Implementation of the 'Phone Type' enum.

    The phone type.

    Attributes:
        FAX: TODO: type description here.
        HOME: TODO: type description here.
        MOBILE: TODO: type description here.
        OTHER: TODO: type description here.
        PAGER: TODO: type description here.

    """
    FAX = 'FAX'

    HOME = 'HOME'

    MOBILE = 'MOBILE'

    OTHER = 'OTHER'

    PAGER = 'PAGER'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class LinkHTTPMethod(object):

    """Implementation of the 'Link HTTP Method' enum.

    The HTTP method required to make the related call.

    Attributes:
        GET: TODO: type description here.
        POST: TODO: type description here.
        PUT: TODO: type description here.
        DELETE: TODO: type description here.
        HEAD: TODO: type description here.
        CONNECT: TODO: type description here.
        OPTIONS: TODO: type description here.
        PATCH: TODO: type description here.

    """
    GET = 'GET'

    POST = 'POST'

    PUT = 'PUT'

    DELETE = 'DELETE'

    HEAD = 'HEAD'

    CONNECT = 'CONNECT'

    OPTIONS = 'OPTIONS'

    PATCH = 'PATCH'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.order_confirm_application_context import OrderConfirmApplicationContext
from paypalserversdk.models.payment_source import PaymentSource


class ConfirmOrderRequest(object):

    """Implementation of the 'Confirm Order Request' model.

    Payer confirms the intent to pay for the Order using the provided payment
    source.

    Attributes:
        payment_source (PaymentSource): The payment source definition.
        processing_instruction (ProcessingInstruction): The instruction to
            process an order.
        application_context (OrderConfirmApplicationContext): Customizes the
            payer confirmation experience.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "payment_source": 'payment_source',
        "processing_instruction": 'processing_instruction',
        "application_context": 'application_context'
    }

    _optionals = [
        'processing_instruction',
        'application_context',
    ]

    def __init__(self,
                 payment_source=None,
                 processing_instruction='NO_INSTRUCTION',
                 application_context=APIHelper.SKIP):
        """Constructor for the ConfirmOrderRequest class"""

        # Initialize members of the class
        self.payment_source = payment_source 
        self.processing_instruction = processing_instruction 
        if application_context is not APIHelper.SKIP:
            self.application_context = application_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        payment_source = PaymentSource.from_dictionary(dictionary.get('payment_source')) if dictionary.get('payment_source') else None
        processing_instruction = dictionary.get("processing_instruction") if dictionary.get("processing_instruction") else 'NO_INSTRUCTION'
        application_context = OrderConfirmApplicationContext.from_dictionary(dictionary.get('application_context')) if 'application_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(payment_source,
                   processing_instruction,
                   application_context)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.order_tracker_item import OrderTrackerItem


class OrderTrackerResponse(object):

    """Implementation of the 'Order Tracker Response' model.

    The tracking response on creation of tracker.

    Attributes:
        id (str): The tracker id.
        status (OrderTrackerStatus): The status of the item shipment.
        items (List[OrderTrackerItem]): An array of details of items in the
            shipment.
        links (List[LinkDescription]): An array of request-related HATEOAS
            links.
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "id": 'id',
        "status": 'status',
        "items": 'items',
        "links": 'links',
        "create_time": 'create_time',
        "update_time": 'update_time'
    }

    _optionals = [
        'id',
        'status',
        'items',
        'links',
        'create_time',
        'update_time',
    ]

    def __init__(self,
                 id=APIHelper.SKIP,
                 status=APIHelper.SKIP,
                 items=APIHelper.SKIP,
                 links=APIHelper.SKIP,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP):
        """Constructor for the OrderTrackerResponse class"""

        # Initialize members of the class
        if id is not APIHelper.SKIP:
            self.id = id 
        if status is not APIHelper.SKIP:
            self.status = status 
        if items is not APIHelper.SKIP:
            self.items = items 
        if links is not APIHelper.SKIP:
            self.links = links 
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        status = dictionary.get("status") if dictionary.get("status") else APIHelper.SKIP
        items = None
        if dictionary.get('items') is not None:
            items = [OrderTrackerItem.from_dictionary(x) for x in dictionary.get('items')]
        else:
            items = APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(id,
                   status,
                   items,
                   links,
                   create_time,
                   update_time)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.authorization_status_details import AuthorizationStatusDetails


class AuthorizationStatusWithDetails(object):

    """Implementation of the 'Authorization Status With Details' model.

    The status fields and status details for an authorized payment.

    Attributes:
        status (AuthorizationStatus): The status for the authorized payment.
        status_details (AuthorizationStatusDetails): The details of the
            authorized payment status.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "status": 'status',
        "status_details": 'status_details'
    }

    _optionals = [
        'status',
        'status_details',
    ]

    def __init__(self,
                 status=APIHelper.SKIP,
                 status_details=APIHelper.SKIP):
        """Constructor for the AuthorizationStatusWithDetails class"""

        # Initialize members of the class
        if status is not APIHelper.SKIP:
            self.status = status 
        if status_details is not APIHelper.SKIP:
            self.status_details = status_details 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        status = dictionary.get("status") if dictionary.get("status") else APIHelper.SKIP
        status_details = AuthorizationStatusDetails.from_dictionary(dictionary.get('status_details')) if 'status_details' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(status,
                   status_details)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.order_authorize_response_payment_source import OrderAuthorizeResponsePaymentSource
from paypalserversdk.models.payer import Payer
from paypalserversdk.models.purchase_unit import PurchaseUnit


class OrderAuthorizeResponse(object):

    """Implementation of the 'Order Authorize Response' model.

    TODO: type model description here.

    Attributes:
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>
        id (str): The ID of the order.
        payment_source (OrderAuthorizeResponsePaymentSource): The payment
            source used to fund the payment.
        intent (CheckoutPaymentIntent): The intent to either capture payment
            immediately or authorize a payment for an order after order
            creation.
        processing_instruction (ProcessingInstruction): The instruction to
            process an order.
        payer (Payer): TODO: type description here.
        purchase_units (List[PurchaseUnit]): An array of purchase units. Each
            purchase unit establishes a contract between a customer and
            merchant. Each purchase unit represents either a full or partial
            order that the customer intends to purchase from the merchant.
        status (OrderStatus): The order status.
        links (List[LinkDescription]): An array of request-related [HATEOAS
            links](/api/rest/responses/#hateoas-links) that are either
            relevant to the issue by providing additional information or
            offering potential resolutions.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "create_time": 'create_time',
        "update_time": 'update_time',
        "id": 'id',
        "payment_source": 'payment_source',
        "intent": 'intent',
        "processing_instruction": 'processing_instruction',
        "payer": 'payer',
        "purchase_units": 'purchase_units',
        "status": 'status',
        "links": 'links'
    }

    _optionals = [
        'create_time',
        'update_time',
        'id',
        'payment_source',
        'intent',
        'processing_instruction',
        'payer',
        'purchase_units',
        'status',
        'links',
    ]

    def __init__(self,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 payment_source=APIHelper.SKIP,
                 intent=APIHelper.SKIP,
                 processing_instruction='NO_INSTRUCTION',
                 payer=APIHelper.SKIP,
                 purchase_units=APIHelper.SKIP,
                 status=APIHelper.SKIP,
                 links=APIHelper.SKIP):
        """Constructor for the OrderAuthorizeResponse class"""

        # Initialize members of the class
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 
        if id is not APIHelper.SKIP:
            self.id = id 
        if payment_source is not APIHelper.SKIP:
            self.payment_source = payment_source 
        if intent is not APIHelper.SKIP:
            self.intent = intent 
        self.processing_instruction = processing_instruction 
        if payer is not APIHelper.SKIP:
            self.payer = payer 
        if purchase_units is not APIHelper.SKIP:
            self.purchase_units = purchase_units 
        if status is not APIHelper.SKIP:
            self.status = status 
        if links is not APIHelper.SKIP:
            self.links = links 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        payment_source = OrderAuthorizeResponsePaymentSource.from_dictionary(dictionary.get('payment_source')) if 'payment_source' in dictionary.keys() else APIHelper.SKIP
        intent = dictionary.get("intent") if dictionary.get("intent") else APIHelper.SKIP
        processing_instruction = dictionary.get("processing_instruction") if dictionary.get("processing_instruction") else 'NO_INSTRUCTION'
        payer = Payer.from_dictionary(dictionary.get('payer')) if 'payer' in dictionary.keys() else APIHelper.SKIP
        purchase_units = None
        if dictionary.get('purchase_units') is not None:
            purchase_units = [PurchaseUnit.from_dictionary(x) for x in dictionary.get('purchase_units')]
        else:
            purchase_units = APIHelper.SKIP
        status = dictionary.get("status") if dictionary.get("status") else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        # Return an object of this model
        return cls(create_time,
                   update_time,
                   id,
                   payment_source,
                   intent,
                   processing_instruction,
                   payer,
                   purchase_units,
                   status,
                   links)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.logger.configuration.api_logging_configuration import (
    ApiLoggingConfiguration,
    ApiRequestLoggingConfiguration,
    ApiResponseLoggingConfiguration)


class RequestLoggingConfiguration(ApiRequestLoggingConfiguration):

    def __init__(self, log_body=False, log_headers=False, headers_to_include=None,
                 headers_to_exclude=None, headers_to_unmask=None,
                 include_query_in_path=False):
        """Default constructor.

        Args:
            log_body (bool): Controls the logging of the request body.
            log_headers (bool): Controls the logging of request headers.
            headers_to_include (List[str]): Includes only specified headers in the log output.
            headers_to_exclude (List[str]): Excludes specified headers from the log output.
            headers_to_unmask (List[str]): Logs specified headers without masking, revealing their actual values.
            include_query_in_path (bool): Determines whether to include query parameters in the logged request path.
        """
        super(RequestLoggingConfiguration, self).__init__(log_body, log_headers, headers_to_include,
                                                          headers_to_exclude, headers_to_unmask,
                                                          include_query_in_path)

    def clone_with(self, log_body=None, log_headers=None, headers_to_include=None,
                   headers_to_exclude=None, headers_to_unmask=None, include_query_in_path=False):
        log_body = log_body or self.log_body
        log_headers = log_headers or self.log_headers
        headers_to_include = headers_to_include or self.headers_to_include
        headers_to_exclude = headers_to_exclude or self.headers_to_exclude
        headers_to_unmask = headers_to_unmask or self.headers_to_unmask
        include_query_in_path = include_query_in_path or self.include_query_in_path

        return RequestLoggingConfiguration(log_body, log_headers, headers_to_include,
                                           headers_to_exclude, headers_to_unmask, include_query_in_path)


class ResponseLoggingConfiguration(ApiResponseLoggingConfiguration):

    def __init__(self, log_body=False, log_headers=False, headers_to_include=None,
                 headers_to_exclude=None, headers_to_unmask=None):
        """Default constructor.

        Args:
            log_body (bool): Controls the logging of the request body.
            log_headers (bool): Controls the logging of request headers.
            headers_to_include (List[str]): Includes only specified headers in the log output.
            headers_to_exclude (List[str]): Excludes specified headers from the log output.
            headers_to_unmask (List[str]): Logs specified headers without masking, revealing their actual values.
        """

        super(ResponseLoggingConfiguration, self).__init__(log_body, log_headers, headers_to_include,
                                                           headers_to_exclude, headers_to_unmask)

    def clone_with(self, log_body=None, log_headers=None, headers_to_include=None,
                   headers_to_exclude=None, headers_to_unmask=None):
        log_body = log_body or self.log_body
        log_headers = log_headers or self.log_headers
        headers_to_include = headers_to_include or self.headers_to_include
        headers_to_exclude = headers_to_exclude or self.headers_to_exclude
        headers_to_unmask = headers_to_unmask or self.headers_to_unmask

        return ResponseLoggingConfiguration(log_body, log_headers, headers_to_include,
                                            headers_to_exclude, headers_to_unmask)


class LoggingConfiguration(ApiLoggingConfiguration):

    def __init__(self, logger=None, log_level=None, mask_sensitive_headers=True,
                 request_logging_config=RequestLoggingConfiguration(),
                 response_logging_config=ResponseLoggingConfiguration()):
        """Default constructor.

        Args:
            logger (Logger): The logging implementation to log with.
            log_level (LogLevel): The log level to apply to the log message.
            mask_sensitive_headers (bool): Flag to control masking of sensitive headers.
            request_logging_config (RequestLoggingConfiguration): The API request logging configuration.
            response_logging_config (ResponseLoggingConfiguration): The API response logging configuration.
        """
        request_logging_config = request_logging_config or RequestLoggingConfiguration()
        response_logging_config = response_logging_config or ResponseLoggingConfiguration()
        super(LoggingConfiguration, self).__init__(logger, log_level, mask_sensitive_headers,
                                                   request_logging_config, response_logging_config)

    def clone_with(self, logger=None, log_level=None, mask_sensitive_headers=None,
                   request_logging_config=None, response_logging_config=None):
        logger = logger or self.logger
        log_level = log_level or self.log_level
        mask_sensitive_headers = mask_sensitive_headers or self.mask_sensitive_headers
        request_logging_config = request_logging_config or self.request_logging_config
        response_logging_config = response_logging_config or self.response_logging_config

        return LoggingConfiguration(logger, log_level, mask_sensitive_headers,
                                    request_logging_config, response_logging_config)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class OrderApplicationContextLandingPage(object):

    """Implementation of the 'Order Application Context Landing Page' enum.

    DEPRECATED. DEPRECATED. The type of landing page to show on the PayPal
    site for customer checkout.  The fields in `application_context` are now
    available in the `experience_context` object under the `payment_source`
    which supports them (eg.
    `payment_source.paypal.experience_context.landing_page`). Please specify
    this field in the `experience_context` object instead of the
    `application_context` object.

    Attributes:
        LOGIN: TODO: type description here.
        BILLING: TODO: type description here.
        NO_PREFERENCE: TODO: type description here.

    """
    LOGIN = 'LOGIN'

    BILLING = 'BILLING'

    NO_PREFERENCE = 'NO_PREFERENCE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.platform_fee import PlatformFee


class CapturePaymentInstruction(object):

    """Implementation of the 'Capture Payment Instruction' model.

    Any additional payment instructions to be consider during payment
    processing. This processing instruction is applicable for Capturing an
    order or Authorizing an Order.

    Attributes:
        platform_fees (List[PlatformFee]): An array of platform or partner
            fees, commissions, or brokerage fees that associated with the
            captured payment.
        disbursement_mode (DisbursementMode): The funds that are held on
            behalf of the merchant.
        payee_receivable_fx_rate_id (str): FX identifier generated returned by
            PayPal to be used for payment processing in order to honor FX rate
            (for eligible integrations) to be used when amount is
            settled/received into the payee account.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "platform_fees": 'platform_fees',
        "disbursement_mode": 'disbursement_mode',
        "payee_receivable_fx_rate_id": 'payee_receivable_fx_rate_id'
    }

    _optionals = [
        'platform_fees',
        'disbursement_mode',
        'payee_receivable_fx_rate_id',
    ]

    def __init__(self,
                 platform_fees=APIHelper.SKIP,
                 disbursement_mode='INSTANT',
                 payee_receivable_fx_rate_id=APIHelper.SKIP):
        """Constructor for the CapturePaymentInstruction class"""

        # Initialize members of the class
        if platform_fees is not APIHelper.SKIP:
            self.platform_fees = platform_fees 
        self.disbursement_mode = disbursement_mode 
        if payee_receivable_fx_rate_id is not APIHelper.SKIP:
            self.payee_receivable_fx_rate_id = payee_receivable_fx_rate_id 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        platform_fees = None
        if dictionary.get('platform_fees') is not None:
            platform_fees = [PlatformFee.from_dictionary(x) for x in dictionary.get('platform_fees')]
        else:
            platform_fees = APIHelper.SKIP
        disbursement_mode = dictionary.get("disbursement_mode") if dictionary.get("disbursement_mode") else 'INSTANT'
        payee_receivable_fx_rate_id = dictionary.get("payee_receivable_fx_rate_id") if dictionary.get("payee_receivable_fx_rate_id") else APIHelper.SKIP
        # Return an object of this model
        return cls(platform_fees,
                   disbursement_mode,
                   payee_receivable_fx_rate_id)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.order_application_context import OrderApplicationContext
from paypalserversdk.models.payer import Payer
from paypalserversdk.models.payment_source import PaymentSource
from paypalserversdk.models.purchase_unit_request import PurchaseUnitRequest


class OrderRequest(object):

    """Implementation of the 'Order Request' model.

    The order request details.

    Attributes:
        intent (CheckoutPaymentIntent): The intent to either capture payment
            immediately or authorize a payment for an order after order
            creation.
        payer (Payer): TODO: type description here.
        purchase_units (List[PurchaseUnitRequest]): An array of purchase
            units. Each purchase unit establishes a contract between a payer
            and the payee. Each purchase unit represents either a full or
            partial order that the payer intends to purchase from the payee.
        payment_source (PaymentSource): The payment source definition.
        application_context (OrderApplicationContext): Customizes the payer
            experience during the approval process for the payment with
            PayPal.<blockquote><strong>Note:</strong> Partners and
            Marketplaces might configure <code>brand_name</code> and
            <code>shipping_preference</code> during partner account setup,
            which overrides the request values.</blockquote>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "intent": 'intent',
        "purchase_units": 'purchase_units',
        "payer": 'payer',
        "payment_source": 'payment_source',
        "application_context": 'application_context'
    }

    _optionals = [
        'payer',
        'payment_source',
        'application_context',
    ]

    def __init__(self,
                 intent=None,
                 purchase_units=None,
                 payer=APIHelper.SKIP,
                 payment_source=APIHelper.SKIP,
                 application_context=APIHelper.SKIP):
        """Constructor for the OrderRequest class"""

        # Initialize members of the class
        self.intent = intent 
        if payer is not APIHelper.SKIP:
            self.payer = payer 
        self.purchase_units = purchase_units 
        if payment_source is not APIHelper.SKIP:
            self.payment_source = payment_source 
        if application_context is not APIHelper.SKIP:
            self.application_context = application_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        intent = dictionary.get("intent") if dictionary.get("intent") else None
        purchase_units = None
        if dictionary.get('purchase_units') is not None:
            purchase_units = [PurchaseUnitRequest.from_dictionary(x) for x in dictionary.get('purchase_units')]
        payer = Payer.from_dictionary(dictionary.get('payer')) if 'payer' in dictionary.keys() else APIHelper.SKIP
        payment_source = PaymentSource.from_dictionary(dictionary.get('payment_source')) if 'payment_source' in dictionary.keys() else APIHelper.SKIP
        application_context = OrderApplicationContext.from_dictionary(dictionary.get('application_context')) if 'application_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(intent,
                   purchase_units,
                   payer,
                   payment_source,
                   application_context)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money


class ReauthorizeRequest(object):

    """Implementation of the 'Reauthorize Request' model.

    Reauthorizes an authorized PayPal account payment, by ID. To ensure that
    funds are still available, reauthorize a payment after its initial
    three-day honor period expires. You can reauthorize a payment only once
    from days four to 29.<br/><br/>If 30 days have transpired since the date
    of the original authorization, you must create an authorized payment
    instead of reauthorizing the original authorized payment.<br/><br/>A
    reauthorized payment itself has a new honor period of three
    days.<br/><br/>You can reauthorize an authorized payment once. The allowed
    amount depends on context and geography, for example in US it is up to
    115% of the original authorized amount, not to exceed an increase of $75
    USD.<br/><br/>Supports only the `amount` request
    parameter.<blockquote><strong>Note:</strong> This request is currently not
    supported for Partner use cases.</blockquote>

    Attributes:
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount'
    }

    _optionals = [
        'amount',
    ]

    def __init__(self,
                 amount=APIHelper.SKIP):
        """Constructor for the ReauthorizeRequest class"""

        # Initialize members of the class
        if amount is not APIHelper.SKIP:
            self.amount = amount 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(amount)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class StandardEntryClassCode(object):

    """Implementation of the 'Standard Entry Class Code' enum.

    NACHA (the regulatory body governing the ACH network) requires that API
    callers (merchants, partners) obtain the consumer’s explicit authorization
    before initiating a transaction. To stay compliant, you’ll need to make
    sure that you retain a compliant authorization for each transaction that
    you originate to the ACH Network using this API. ACH transactions are
    categorized (using SEC codes) by how you capture authorization from the
    Receiver (the person whose bank account is being debited or credited).
    PayPal supports the following SEC codes.

    Attributes:
        TEL: TODO: type description here.
        WEB: TODO: type description here.
        CCD: TODO: type description here.
        PPD: TODO: type description here.

    """
    TEL = 'TEL'

    WEB = 'WEB'

    CCD = 'CCD'

    PPD = 'PPD'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CardVerificationProcessorResponse(object):

    """Implementation of the 'Card Verification Processor Response' model.

    The processor response information for payment requests, such as direct
    credit card transactions.

    Attributes:
        avs_code (AVSCode): The address verification code for Visa, Discover,
            Mastercard, or American Express transactions.
        cvv_code (CVVCode): The card verification value code for for Visa,
            Discover, Mastercard, or American Express.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "avs_code": 'avs_code',
        "cvv_code": 'cvv_code'
    }

    _optionals = [
        'avs_code',
        'cvv_code',
    ]

    def __init__(self,
                 avs_code=APIHelper.SKIP,
                 cvv_code=APIHelper.SKIP):
        """Constructor for the CardVerificationProcessorResponse class"""

        # Initialize members of the class
        if avs_code is not APIHelper.SKIP:
            self.avs_code = avs_code 
        if cvv_code is not APIHelper.SKIP:
            self.cvv_code = cvv_code 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        avs_code = dictionary.get("avs_code") if dictionary.get("avs_code") else APIHelper.SKIP
        cvv_code = dictionary.get("cvv_code") if dictionary.get("cvv_code") else APIHelper.SKIP
        # Return an object of this model
        return cls(avs_code,
                   cvv_code)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.amount_breakdown import AmountBreakdown


class AmountWithBreakdown(object):

    """Implementation of the 'Amount With Breakdown' model.

    The total order amount with an optional breakdown that provides details,
    such as the total item amount, total tax amount, shipping, handling,
    insurance, and discounts, if any.<br/>If you specify `amount.breakdown`,
    the amount equals `item_total` plus `tax_total` plus `shipping` plus
    `handling` plus `insurance` minus `shipping_discount` minus
    discount.<br/>The amount must be a positive number. For listed of
    supported currencies and decimal precision, see the PayPal REST APIs <a
    href="/docs/integration/direct/rest/currency-codes/">Currency Codes</a>.

    Attributes:
        currency_code (str): The [three-character ISO-4217 currency
            code](/api/rest/reference/currency-codes/) that identifies the
            currency.
        value (str): The value, which might be:<ul><li>An integer for
            currencies like `JPY` that are not typically fractional.</li><li>A
            decimal fraction for currencies like `TND` that are subdivided
            into thousandths.</li></ul>For the required number of decimal
            places for a currency code, see [Currency
            Codes](/api/rest/reference/currency-codes/).
        breakdown (AmountBreakdown): The breakdown of the amount. Breakdown
            provides details such as total item amount, total tax amount,
            shipping, handling, insurance, and discounts, if any.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "currency_code": 'currency_code',
        "value": 'value',
        "breakdown": 'breakdown'
    }

    _optionals = [
        'breakdown',
    ]

    def __init__(self,
                 currency_code=None,
                 value=None,
                 breakdown=APIHelper.SKIP):
        """Constructor for the AmountWithBreakdown class"""

        # Initialize members of the class
        self.currency_code = currency_code 
        self.value = value 
        if breakdown is not APIHelper.SKIP:
            self.breakdown = breakdown 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        currency_code = dictionary.get("currency_code") if dictionary.get("currency_code") else None
        value = dictionary.get("value") if dictionary.get("value") else None
        breakdown = AmountBreakdown.from_dictionary(dictionary.get('breakdown')) if 'breakdown' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(currency_code,
                   value,
                   breakdown)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.exchange_rate import ExchangeRate
from paypalserversdk.models.money import Money
from paypalserversdk.models.platform_fee import PlatformFee


class SellerReceivableBreakdown(object):

    """Implementation of the 'Seller Receivable Breakdown' model.

    The detailed breakdown of the capture activity. This is not available for
    transactions that are in pending state.

    Attributes:
        gross_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        paypal_fee (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        paypal_fee_in_receivable_currency (Money): The currency and amount for
            a financial transaction, such as a balance or payment due.
        net_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        receivable_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        exchange_rate (ExchangeRate): The exchange rate that determines the
            amount to convert from one currency to another currency.
        platform_fees (List[PlatformFee]): An array of platform or partner
            fees, commissions, or brokerage fees that associated with the
            captured payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "gross_amount": 'gross_amount',
        "paypal_fee": 'paypal_fee',
        "paypal_fee_in_receivable_currency": 'paypal_fee_in_receivable_currency',
        "net_amount": 'net_amount',
        "receivable_amount": 'receivable_amount',
        "exchange_rate": 'exchange_rate',
        "platform_fees": 'platform_fees'
    }

    _optionals = [
        'paypal_fee',
        'paypal_fee_in_receivable_currency',
        'net_amount',
        'receivable_amount',
        'exchange_rate',
        'platform_fees',
    ]

    def __init__(self,
                 gross_amount=None,
                 paypal_fee=APIHelper.SKIP,
                 paypal_fee_in_receivable_currency=APIHelper.SKIP,
                 net_amount=APIHelper.SKIP,
                 receivable_amount=APIHelper.SKIP,
                 exchange_rate=APIHelper.SKIP,
                 platform_fees=APIHelper.SKIP):
        """Constructor for the SellerReceivableBreakdown class"""

        # Initialize members of the class
        self.gross_amount = gross_amount 
        if paypal_fee is not APIHelper.SKIP:
            self.paypal_fee = paypal_fee 
        if paypal_fee_in_receivable_currency is not APIHelper.SKIP:
            self.paypal_fee_in_receivable_currency = paypal_fee_in_receivable_currency 
        if net_amount is not APIHelper.SKIP:
            self.net_amount = net_amount 
        if receivable_amount is not APIHelper.SKIP:
            self.receivable_amount = receivable_amount 
        if exchange_rate is not APIHelper.SKIP:
            self.exchange_rate = exchange_rate 
        if platform_fees is not APIHelper.SKIP:
            self.platform_fees = platform_fees 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        gross_amount = Money.from_dictionary(dictionary.get('gross_amount')) if dictionary.get('gross_amount') else None
        paypal_fee = Money.from_dictionary(dictionary.get('paypal_fee')) if 'paypal_fee' in dictionary.keys() else APIHelper.SKIP
        paypal_fee_in_receivable_currency = Money.from_dictionary(dictionary.get('paypal_fee_in_receivable_currency')) if 'paypal_fee_in_receivable_currency' in dictionary.keys() else APIHelper.SKIP
        net_amount = Money.from_dictionary(dictionary.get('net_amount')) if 'net_amount' in dictionary.keys() else APIHelper.SKIP
        receivable_amount = Money.from_dictionary(dictionary.get('receivable_amount')) if 'receivable_amount' in dictionary.keys() else APIHelper.SKIP
        exchange_rate = ExchangeRate.from_dictionary(dictionary.get('exchange_rate')) if 'exchange_rate' in dictionary.keys() else APIHelper.SKIP
        platform_fees = None
        if dictionary.get('platform_fees') is not None:
            platform_fees = [PlatformFee.from_dictionary(x) for x in dictionary.get('platform_fees')]
        else:
            platform_fees = APIHelper.SKIP
        # Return an object of this model
        return cls(gross_amount,
                   paypal_fee,
                   paypal_fee_in_receivable_currency,
                   net_amount,
                   receivable_amount,
                   exchange_rate,
                   platform_fees)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.experience_context import ExperienceContext


class GiropayPaymentRequest(object):

    """Implementation of the 'Giropay Payment Request' model.

    Information needed to pay using giropay.

    Attributes:
        name (str): The full name representation like Mr J Smith.
        country_code (str): The [two-character ISO 3166-1
            code](/api/rest/reference/country-codes/) that identifies the
            country or region.<blockquote><strong>Note:</strong> The country
            code for Great Britain is <code>GB</code> and not <code>UK</code>
            as used in the top-level domain names for that country. Use the
            `C2` country code for China worldwide for comparable uncontrolled
            price (CUP) method, bank card, and cross-border
            transactions.</blockquote>
        experience_context (ExperienceContext): Customizes the payer
            experience during the approval process for the payment.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "name": 'name',
        "country_code": 'country_code',
        "experience_context": 'experience_context'
    }

    _optionals = [
        'experience_context',
    ]

    def __init__(self,
                 name=None,
                 country_code=None,
                 experience_context=APIHelper.SKIP):
        """Constructor for the GiropayPaymentRequest class"""

        # Initialize members of the class
        self.name = name 
        self.country_code = country_code 
        if experience_context is not APIHelper.SKIP:
            self.experience_context = experience_context 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        name = dictionary.get("name") if dictionary.get("name") else None
        country_code = dictionary.get("country_code") if dictionary.get("country_code") else None
        experience_context = ExperienceContext.from_dictionary(dictionary.get('experience_context')) if 'experience_context' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(name,
                   country_code,
                   experience_context)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.money import Money
from paypalserversdk.models.refund_payment_instruction import RefundPaymentInstruction


class RefundRequest(object):

    """Implementation of the 'Refund Request' model.

    Refunds a captured payment, by ID. For a full refund, include an empty
    request body. For a partial refund, include an <code>amount</code> object
    in the request body.

    Attributes:
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        custom_id (str): The API caller-provided external ID. Used to
            reconcile API caller-initiated transactions with PayPal
            transactions. Appears in transaction and settlement reports. The
            pattern is defined by an external party and supports Unicode.
        invoice_id (str): The API caller-provided external invoice ID for this
            order. The pattern is defined by an external party and supports
            Unicode.
        note_to_payer (str): The reason for the refund. Appears in both the
            payer's transaction history and the emails that the payer
            receives. The pattern is defined by an external party and supports
            Unicode.
        payment_instruction (RefundPaymentInstruction): Any additional
            payments instructions during refund payment processing. This
            object is only applicable to merchants that have been enabled for
            PayPal Commerce Platform for Marketplaces and Platforms
            capability. Please speak to your account manager if you want to
            use this capability.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "amount": 'amount',
        "custom_id": 'custom_id',
        "invoice_id": 'invoice_id',
        "note_to_payer": 'note_to_payer',
        "payment_instruction": 'payment_instruction'
    }

    _optionals = [
        'amount',
        'custom_id',
        'invoice_id',
        'note_to_payer',
        'payment_instruction',
    ]

    def __init__(self,
                 amount=APIHelper.SKIP,
                 custom_id=APIHelper.SKIP,
                 invoice_id=APIHelper.SKIP,
                 note_to_payer=APIHelper.SKIP,
                 payment_instruction=APIHelper.SKIP):
        """Constructor for the RefundRequest class"""

        # Initialize members of the class
        if amount is not APIHelper.SKIP:
            self.amount = amount 
        if custom_id is not APIHelper.SKIP:
            self.custom_id = custom_id 
        if invoice_id is not APIHelper.SKIP:
            self.invoice_id = invoice_id 
        if note_to_payer is not APIHelper.SKIP:
            self.note_to_payer = note_to_payer 
        if payment_instruction is not APIHelper.SKIP:
            self.payment_instruction = payment_instruction 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        custom_id = dictionary.get("custom_id") if dictionary.get("custom_id") else APIHelper.SKIP
        invoice_id = dictionary.get("invoice_id") if dictionary.get("invoice_id") else APIHelper.SKIP
        note_to_payer = dictionary.get("note_to_payer") if dictionary.get("note_to_payer") else APIHelper.SKIP
        payment_instruction = RefundPaymentInstruction.from_dictionary(dictionary.get('payment_instruction')) if 'payment_instruction' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(amount,
                   custom_id,
                   invoice_id,
                   note_to_payer,
                   payment_instruction)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class ProcessingInstruction(object):

    """Implementation of the 'Processing Instruction' enum.

    The instruction to process an order.

    Attributes:
        ORDER_COMPLETE_ON_PAYMENT_APPROVAL: TODO: type description here.
        NO_INSTRUCTION: TODO: type description here.

    """
    ORDER_COMPLETE_ON_PAYMENT_APPROVAL = 'ORDER_COMPLETE_ON_PAYMENT_APPROVAL'

    NO_INSTRUCTION = 'NO_INSTRUCTION'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class TaxInfo(object):

    """Implementation of the 'Tax Info' model.

    The tax ID of the customer. The customer is also known as the payer. Both
    `tax_id` and `tax_id_type` are required.

    Attributes:
        tax_id (str): The customer's tax ID value.
        tax_id_type (TaxIdType): The customer's tax ID type.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tax_id": 'tax_id',
        "tax_id_type": 'tax_id_type'
    }

    def __init__(self,
                 tax_id=None,
                 tax_id_type=None):
        """Constructor for the TaxInfo class"""

        # Initialize members of the class
        self.tax_id = tax_id 
        self.tax_id_type = tax_id_type 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        tax_id = dictionary.get("tax_id") if dictionary.get("tax_id") else None
        tax_id_type = dictionary.get("tax_id_type") if dictionary.get("tax_id_type") else None
        # Return an object of this model
        return cls(tax_id,
                   tax_id_type)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.address import Address
from paypalserversdk.models.name import Name
from paypalserversdk.models.pay_pal_wallet_attributes_response import PayPalWalletAttributesResponse
from paypalserversdk.models.phone_number import PhoneNumber
from paypalserversdk.models.tax_info import TaxInfo


class PayPalWalletResponse(object):

    """Implementation of the 'PayPal Wallet Response' model.

    The PayPal Wallet response.

    Attributes:
        email_address (str): The internationalized email
            address.<blockquote><strong>Note:</strong> Up to 64 characters are
            allowed before and 255 characters are allowed after the
            <code>@</code> sign. However, the generally accepted maximum
            length for an email address is 254 characters. The pattern
            verifies that an unquoted <code>@</code> sign
            exists.</blockquote>
        account_id (str): The PayPal payer ID, which is a masked version of
            the PayPal account number intended for use with third parties. The
            account number is reversibly encrypted and a proprietary variant
            of Base32 is used to encode the result.
        account_status (PayPalWalletAccountVerificationStatus): The account
            status indicates whether the buyer has verified the financial
            details associated with their PayPal account.
        name (Name): The name of the party.
        phone_type (PhoneType): The phone type.
        phone_number (PhoneNumber): The phone number in its canonical
            international [E.164 numbering plan
            format](https://www.itu.int/rec/T-REC-E.164/en).
        birth_date (str): The stand-alone date, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). To
            represent special legal values, such as a date of birth, you
            should use dates with no associated time or time-zone data.
            Whenever possible, use the standard `date_time` type. This regular
            expression does not validate all dates. For example, February 31
            is valid and nothing is known about leap years.
        business_name (str): The business name of the PayPal account holder
            (populated for business accounts only)
        tax_info (TaxInfo): The tax ID of the customer. The customer is also
            known as the payer. Both `tax_id` and `tax_id_type` are required.
        address (Address): The portable international postal address. Maps to
            [AddressValidationMetadata](https://github.com/googlei18n/libaddres
            sinput/wiki/AddressValidationMetadata) and HTML 5.1 [Autofilling
            form controls: the autocomplete
            attribute](https://www.w3.org/TR/html51/sec-forms.html#autofilling-
            form-controls-the-autocomplete-attribute).
        attributes (PayPalWalletAttributesResponse): Additional attributes
            associated with the use of a PayPal Wallet.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "email_address": 'email_address',
        "account_id": 'account_id',
        "account_status": 'account_status',
        "name": 'name',
        "phone_type": 'phone_type',
        "phone_number": 'phone_number',
        "birth_date": 'birth_date',
        "business_name": 'business_name',
        "tax_info": 'tax_info',
        "address": 'address',
        "attributes": 'attributes'
    }

    _optionals = [
        'email_address',
        'account_id',
        'account_status',
        'name',
        'phone_type',
        'phone_number',
        'birth_date',
        'business_name',
        'tax_info',
        'address',
        'attributes',
    ]

    def __init__(self,
                 email_address=APIHelper.SKIP,
                 account_id=APIHelper.SKIP,
                 account_status=APIHelper.SKIP,
                 name=APIHelper.SKIP,
                 phone_type=APIHelper.SKIP,
                 phone_number=APIHelper.SKIP,
                 birth_date=APIHelper.SKIP,
                 business_name=APIHelper.SKIP,
                 tax_info=APIHelper.SKIP,
                 address=APIHelper.SKIP,
                 attributes=APIHelper.SKIP):
        """Constructor for the PayPalWalletResponse class"""

        # Initialize members of the class
        if email_address is not APIHelper.SKIP:
            self.email_address = email_address 
        if account_id is not APIHelper.SKIP:
            self.account_id = account_id 
        if account_status is not APIHelper.SKIP:
            self.account_status = account_status 
        if name is not APIHelper.SKIP:
            self.name = name 
        if phone_type is not APIHelper.SKIP:
            self.phone_type = phone_type 
        if phone_number is not APIHelper.SKIP:
            self.phone_number = phone_number 
        if birth_date is not APIHelper.SKIP:
            self.birth_date = birth_date 
        if business_name is not APIHelper.SKIP:
            self.business_name = business_name 
        if tax_info is not APIHelper.SKIP:
            self.tax_info = tax_info 
        if address is not APIHelper.SKIP:
            self.address = address 
        if attributes is not APIHelper.SKIP:
            self.attributes = attributes 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        email_address = dictionary.get("email_address") if dictionary.get("email_address") else APIHelper.SKIP
        account_id = dictionary.get("account_id") if dictionary.get("account_id") else APIHelper.SKIP
        account_status = dictionary.get("account_status") if dictionary.get("account_status") else APIHelper.SKIP
        name = Name.from_dictionary(dictionary.get('name')) if 'name' in dictionary.keys() else APIHelper.SKIP
        phone_type = dictionary.get("phone_type") if dictionary.get("phone_type") else APIHelper.SKIP
        phone_number = PhoneNumber.from_dictionary(dictionary.get('phone_number')) if 'phone_number' in dictionary.keys() else APIHelper.SKIP
        birth_date = dictionary.get("birth_date") if dictionary.get("birth_date") else APIHelper.SKIP
        business_name = dictionary.get("business_name") if dictionary.get("business_name") else APIHelper.SKIP
        tax_info = TaxInfo.from_dictionary(dictionary.get('tax_info')) if 'tax_info' in dictionary.keys() else APIHelper.SKIP
        address = Address.from_dictionary(dictionary.get('address')) if 'address' in dictionary.keys() else APIHelper.SKIP
        attributes = PayPalWalletAttributesResponse.from_dictionary(dictionary.get('attributes')) if 'attributes' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(email_address,
                   account_id,
                   account_status,
                   name,
                   phone_type,
                   phone_number,
                   birth_date,
                   business_name,
                   tax_info,
                   address,
                   attributes)



__all__ = [
    'activity_timestamps',
    'amount_breakdown',
    'amount_with_breakdown',
    'apple_pay_payment_object',
    'apple_pay_attributes',
    'apple_pay_attributes_response',
    'apple_pay_card_response',
    'apple_pay_decrypted_token_data',
    'apple_pay_payment_data',
    'apple_pay_request',
    'assurance_details',
    'authentication_response',
    'authorization',
    'link_description',
    'authorization_status_with_details',
    'authorization_status_details',
    'authorization_with_additional_data',
    'bancontact_payment_object',
    'bancontact_payment_request',
    'bin_details',
    'blik_payment_object',
    'blik_experience_context',
    'blik_one_click_payment_request',
    'blik_one_click_payment_object',
    'blik_payment_request',
    'blik_level_0_payment_object',
    'capture',
    'capture_status_with_details',
    'capture_status_details',
    'card_attributes',
    'card_attributes_response',
    'card_customer_information',
    'card_experience_context',
    'card_from_request',
    'card_request',
    'card_response',
    'card_stored_credential',
    'card_supplementary_data',
    'card_vault_response',
    'card_verification',
    'cobranded_card',
    'confirm_order_request',
    'customer_information',
    'platform_fee',
    'eps_payment_object',
    'eps_payment_request',
    'error_details',
    'exchange_rate',
    'experience_context',
    'giropay_payment_object',
    'giropay_payment_request',
    'google_pay_wallet_response',
    'google_pay_card_response',
    'google_pay_decrypted_token_data',
    'google_pay_request',
    'ideal_payment_object',
    'ideal_payment_request',
    'item',
    'level_2_card_processing_data',
    'level_3_card_processing_data',
    'line_item',
    'money',
    'my_bank_payment_object',
    'my_bank_payment_request',
    'net_amount_breakdown_item',
    'network_token',
    'network_transaction_reference',
    'order',
    'order_application_context',
    'order_authorize_request',
    'order_authorize_response',
    'order_capture_request',
    'order_confirm_application_context',
    'order_request',
    'order_tracker_request',
    'p_24_payment_object',
    'p_24_payment_request',
    'patch',
    'payee',
    'payee_base',
    'payer',
    'payer_base',
    'payment_collection',
    'payment_instruction',
    'payment_method_preference',
    'payment_source',
    'payment_source_response',
    'pay_pal_wallet',
    'pay_pal_wallet_attributes',
    'pay_pal_wallet_attributes_response',
    'pay_pal_wallet_customer',
    'pay_pal_wallet_experience_context',
    'pay_pal_wallet_response',
    'pay_pal_wallet_vault_response',
    'phone_number',
    'phone_with_type',
    'processor_response',
    'purchase_unit',
    'purchase_unit_request',
    'refund',
    'refund_status_with_details',
    'refund_status_details',
    'seller_payable_breakdown',
    'seller_protection',
    'seller_receivable_breakdown',
    'shipping_details',
    'shipping_option',
    'shipping_with_tracking_details',
    'sofort_payment_object',
    'sofort_payment_request',
    'stored_payment_source',
    'supplementary_data',
    'tax_info',
    'three_d_secure_authentication_response',
    'token',
    'order_tracker_response',
    'order_tracker_item',
    'trustly_payment_object',
    'trustly_payment_request',
    'universal_product_code',
    'vault_instruction',
    'vault_instruction_base',
    'vault_response',
    'venmo_wallet_vault_attributes',
    'venmo_wallet_additional_attributes',
    'venmo_wallet_attributes_response',
    'venmo_wallet_experience_context',
    'venmo_wallet_request',
    'venmo_wallet_response',
    'address',
    'apple_pay_tokenized_card',
    'google_pay_card_attributes',
    'google_pay_request_card',
    'name',
    'order_authorize_request_payment_source',
    'order_authorize_response_payment_source',
    'order_capture_request_payment_source',
    'pay_pal_wallet_customer_request',
    'pay_pal_wallet_vault_instruction',
    'phone_number_with_country_code',
    'shipping_name',
    'vault_customer',
    'venmo_wallet_customer_information',
    'payment_authorization',
    'captured_payment',
    'capture_request',
    'refund_platform_fee',
    'refund_payment_instruction',
    'reauthorize_request',
    'refund_request',
    'related_identifiers',
    'payment_supplementary_data',
    'supplementary_purchase_data',
    'capture_payment_instruction',
    'address_details',
    'apple_pay_payment_token',
    'card_payment_token',
    'card_response_entity',
    'card_verification_details',
    'customer_request',
    'customer_vault_payment_tokens_response',
    'payment_token_request_payment_source',
    'vault_experience_context',
    'network_transaction_reference_entity',
    'payment_token_response_payment_source',
    'payment_token_request',
    'payment_token_response',
    'vault_pay_pal_wallet_request',
    'pay_pal_payment_token',
    'phone',
    'setup_token_request',
    'setup_token_response',
    'vault_token_request',
    'vault_venmo_request',
    'venmo_payment_token',
    'vaulted_digital_wallet',
    'apple_pay_card',
    'card_authentication_response',
    'card_verification_processor_response',
    'payment_token_request_card',
    'setup_token_card_experience_context',
    'setup_token_request_card',
    'setup_token_request_payment_source',
    'setup_token_response_card',
    'setup_token_response_payment_source',
    'vault_venmo_experience_context',
    'vaulted_digital_wallet_shipping_details',
    'o_auth_token',
    'card_brand',
    'card_type',
    'checkout_payment_intent',
    'disbursement_mode',
    'dispute_category',
    'eci_flag',
    'enrollment_status',
    'liability_shift_indicator',
    'order_status',
    'pa_res_status',
    'payee_payment_method_preference',
    'payment_initiator',
    'phone_type',
    'processing_instruction',
    'shipment_carrier',
    'shipping_type',
    'store_in_vault_instruction',
    'stored_payment_source_payment_type',
    'stored_payment_source_usage_type',
    'order_tracker_status',
    'apple_pay_payment_data_type',
    'authorization_incomplete_reason',
    'authorization_status',
    'avs_code',
    'capture_incomplete_reason',
    'capture_status',
    'card_verification_method',
    'cvv_code',
    'fullfillment_type',
    'google_pay_authentication_method',
    'google_pay_payment_method',
    'item_category',
    'link_http_method',
    'order_application_context_landing_page',
    'order_application_context_shipping_preference',
    'order_application_context_user_action',
    'patch_op',
    'payment_advice_code',
    'pay_pal_experience_landing_page',
    'pay_pal_experience_user_action',
    'pay_pal_payment_token_customer_type',
    'pay_pal_payment_token_usage_pattern',
    'pay_pal_payment_token_usage_type',
    'pay_pal_wallet_account_verification_status',
    'pay_pal_wallet_vault_status',
    'processor_response_code',
    'refund_incomplete_reason',
    'refund_status',
    'seller_protection_status',
    'shipping_preference',
    'standard_entry_class_code',
    'tax_id_type',
    'token_type',
    'upc_type',
    'vault_status',
    'venmo_payment_token_customer_type',
    'venmo_payment_token_usage_pattern',
    'venmo_payment_token_usage_type',
    'token_request_type',
    'o_auth_provider_error',
]



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from apimatic_core.authentication.header_auth import HeaderAuth
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.o_auth_token import OAuthToken
from apimatic_core.utilities.auth_helper import AuthHelper
from paypalserversdk.controllers.o_auth_authorization_controller import\
    OAuthAuthorizationController
from paypalserversdk.exceptions.api_exception import APIException


class OAuth2(HeaderAuth):

    @property
    def error_message(self):
        """Display error message on occurrence of authentication failure
        in ClientCredentialsAuth

        """
        return "ClientCredentialsAuth: OAuthToken is undefined or expired."

    def __init__(self, client_credentials_auth_credentials, config):
        self._o_auth_client_id = client_credentials_auth_credentials.o_auth_client_id \
            if client_credentials_auth_credentials is not None else None
        self._o_auth_client_secret = client_credentials_auth_credentials.o_auth_client_secret \
            if client_credentials_auth_credentials is not None else None
        if client_credentials_auth_credentials is not None \
                and isinstance(client_credentials_auth_credentials.o_auth_token, OAuthToken):
            self._o_auth_token = OAuthToken.from_dictionary(
                APIHelper.to_dictionary(client_credentials_auth_credentials.o_auth_token))
        else:
            self._o_auth_token = client_credentials_auth_credentials.o_auth_token \
                if client_credentials_auth_credentials is not None else None
        self._o_auth_clock_skew = client_credentials_auth_credentials.o_auth_clock_skew \
            if client_credentials_auth_credentials is not None else None
        self._o_auth_token_provider = client_credentials_auth_credentials.o_auth_token_provider \
            if client_credentials_auth_credentials is not None else None
        self._o_auth_on_token_update = client_credentials_auth_credentials.o_auth_on_token_update \
            if client_credentials_auth_credentials is not None else None
        self._o_auth_api = OAuthAuthorizationController(config)
        super().__init__(auth_params={})

    def is_valid(self):
        self._o_auth_token = self._get_token_from_provider()
        return (self._o_auth_token and isinstance(self._o_auth_token, OAuthToken)
                and not self.is_token_expired(self._o_auth_token))

    def build_basic_auth_header(self):
        """ Builds the basic auth header for endpoints in the
            OAuth Authorization Controller.

        Returns:
            str: The value of the Authentication header.

        """
        return "Basic {}".format(AuthHelper.get_base64_encoded_value(
            self._o_auth_client_id, self._o_auth_client_secret))

    def fetch_token(self, additional_params=None):
        """ Authorizes the client.

            
            additional_params (dict):  Any additional form parameters.

        Returns:
            OAuthToken: The OAuth token.

        """
        token = self._o_auth_api.request_token(
            {
                'authorization': self.build_basic_auth_header()
            },
            _optional_form_parameters=additional_params
        ).body
        if hasattr(token, 'expires_in'):
            current_utc_timestamp = AuthHelper.get_current_utc_timestamp()
            token.expiry = AuthHelper.get_token_expiry(current_utc_timestamp, token.expires_in)
        return token

    def is_token_expired(self, o_auth_token=None):
        """ Checks if OAuth token has expired.

        Args:
            o_auth_token (OAuthToken): The OAuth token whose expiry is to be checked.

        Returns:
            bool: True if OAuth token has expired, False otherwise.

        """
        if o_auth_token is None:
            return (hasattr(self._o_auth_token, 'expiry')
                    and AuthHelper.is_token_expired(self._o_auth_token.expiry, self._o_auth_clock_skew))

        return (hasattr(o_auth_token, 'expiry')
                and AuthHelper.is_token_expired(o_auth_token.expiry, self._o_auth_clock_skew))

    def apply(self, http_request):
        auth_params = {"Authorization": "Bearer {}".format(self._o_auth_token.access_token)}
        AuthHelper.apply(auth_params, http_request.add_header)

    def _get_token_from_provider(self):
        """ This provides the OAuth Token from either the user configured callbacks or from default provider.
        Returns:
            OAuthToken: The fetched OAuth token.
        """
        if self._o_auth_token is not None and not self.is_token_expired(self._o_auth_token):
            return self._o_auth_token

        if self._o_auth_token_provider is not None:
            o_auth_token = self._o_auth_token_provider(self._o_auth_token, self)
            self._apply_on_token_update_callback(o_auth_token)
            return o_auth_token

        try:
            o_auth_token = self.fetch_token()
            self._apply_on_token_update_callback(o_auth_token)
            return o_auth_token
        except APIException:
            return self._o_auth_token

    def _apply_on_token_update_callback(self, o_auth_token):
        """ This function applies the OAuth token update callback provided by the user.
        """
        if self._o_auth_on_token_update is not None:
            self._o_auth_on_token_update(o_auth_token)


class ClientCredentialsAuthCredentials:

    @property
    def o_auth_client_id(self):
        return self._o_auth_client_id

    @property
    def o_auth_client_secret(self):
        return self._o_auth_client_secret

    @property
    def o_auth_token(self):
        return self._o_auth_token

    @property
    def o_auth_token_provider(self):
        return self._o_auth_token_provider

    @property
    def o_auth_on_token_update(self):
        return self._o_auth_on_token_update

    @property
    def o_auth_clock_skew(self):
        return self._o_auth_clock_skew

    def __init__(self, o_auth_client_id, o_auth_client_secret,
                 o_auth_token=None, o_auth_token_provider=None,
                 o_auth_on_token_update=None, o_auth_clock_skew=None):
        if o_auth_client_id is None:
            raise ValueError('o_auth_client_id cannot be None')
        if o_auth_client_secret is None:
            raise ValueError('o_auth_client_secret cannot be None')
        self._o_auth_client_id = o_auth_client_id
        self._o_auth_client_secret = o_auth_client_secret
        self._o_auth_token = o_auth_token
        self._o_auth_token_provider = o_auth_token_provider
        self._o_auth_on_token_update = o_auth_on_token_update
        self._o_auth_clock_skew = o_auth_clock_skew

    def clone_with(self, o_auth_client_id=None, o_auth_client_secret=None,
                   o_auth_token=None, o_auth_token_provider=None,
                   o_auth_on_token_update=None, o_auth_clock_skew=None):
        return ClientCredentialsAuthCredentials(
            o_auth_client_id or self.o_auth_client_id,
            o_auth_client_secret or self.o_auth_client_secret,
            o_auth_token or self.o_auth_token,
            o_auth_token_provider or self.o_auth_token_provider,
            o_auth_on_token_update or self.o_auth_on_token_update,
            o_auth_clock_skew or self.o_auth_clock_skew)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PatchOp(object):

    """Implementation of the 'Patch Op' enum.

    The operation.

    Attributes:
        ADD: TODO: type description here.
        REMOVE: TODO: type description here.
        REPLACE: TODO: type description here.
        MOVE: TODO: type description here.
        COPY: TODO: type description here.
        TEST: TODO: type description here.

    """
    ADD = 'add'

    REMOVE = 'remove'

    REPLACE = 'replace'

    MOVE = 'move'

    COPY = 'copy'

    TEST = 'test'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class PayPalExperienceLandingPage(object):

    """Implementation of the 'PayPal Experience Landing Page' enum.

    The type of landing page to show on the PayPal site for customer
    checkout.

    Attributes:
        LOGIN: TODO: type description here.
        GUEST_CHECKOUT: TODO: type description here.
        NO_PREFERENCE: TODO: type description here.

    """
    LOGIN = 'LOGIN'

    GUEST_CHECKOUT = 'GUEST_CHECKOUT'

    NO_PREFERENCE = 'NO_PREFERENCE'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.apple_pay_payment_data import ApplePayPaymentData
from paypalserversdk.models.apple_pay_tokenized_card import ApplePayTokenizedCard
from paypalserversdk.models.money import Money


class ApplePayDecryptedTokenData(object):

    """Implementation of the 'Apple Pay Decrypted Token Data' model.

    Information about the Payment data obtained by decrypting Apple Pay
    token.

    Attributes:
        transaction_amount (Money): The currency and amount for a financial
            transaction, such as a balance or payment due.
        tokenized_card (ApplePayTokenizedCard): The payment card to use to
            fund a payment. Can be a credit or debit card.
        device_manufacturer_id (str): Apple Pay Hex-encoded device
            manufacturer identifier. The pattern is defined by an external
            party and supports Unicode.
        payment_data_type (ApplePayPaymentDataType): Indicates the type of
            payment data passed, in case of Non China the payment data is
            3DSECURE and for China it is EMV.
        payment_data (ApplePayPaymentData): Information about the decrypted
            apple pay payment data for the token like cryptogram, eci
            indicator.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "tokenized_card": 'tokenized_card',
        "transaction_amount": 'transaction_amount',
        "device_manufacturer_id": 'device_manufacturer_id',
        "payment_data_type": 'payment_data_type',
        "payment_data": 'payment_data'
    }

    _optionals = [
        'transaction_amount',
        'device_manufacturer_id',
        'payment_data_type',
        'payment_data',
    ]

    def __init__(self,
                 tokenized_card=None,
                 transaction_amount=APIHelper.SKIP,
                 device_manufacturer_id=APIHelper.SKIP,
                 payment_data_type=APIHelper.SKIP,
                 payment_data=APIHelper.SKIP):
        """Constructor for the ApplePayDecryptedTokenData class"""

        # Initialize members of the class
        if transaction_amount is not APIHelper.SKIP:
            self.transaction_amount = transaction_amount 
        self.tokenized_card = tokenized_card 
        if device_manufacturer_id is not APIHelper.SKIP:
            self.device_manufacturer_id = device_manufacturer_id 
        if payment_data_type is not APIHelper.SKIP:
            self.payment_data_type = payment_data_type 
        if payment_data is not APIHelper.SKIP:
            self.payment_data = payment_data 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        tokenized_card = ApplePayTokenizedCard.from_dictionary(dictionary.get('tokenized_card')) if dictionary.get('tokenized_card') else None
        transaction_amount = Money.from_dictionary(dictionary.get('transaction_amount')) if 'transaction_amount' in dictionary.keys() else APIHelper.SKIP
        device_manufacturer_id = dictionary.get("device_manufacturer_id") if dictionary.get("device_manufacturer_id") else APIHelper.SKIP
        payment_data_type = dictionary.get("payment_data_type") if dictionary.get("payment_data_type") else APIHelper.SKIP
        payment_data = ApplePayPaymentData.from_dictionary(dictionary.get('payment_data')) if 'payment_data' in dictionary.keys() else APIHelper.SKIP
        # Return an object of this model
        return cls(tokenized_card,
                   transaction_amount,
                   device_manufacturer_id,
                   payment_data_type,
                   payment_data)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper


class CardVerification(object):

    """Implementation of the 'Card Verification' model.

    The API caller can opt in to verify the card through PayPal offered
    verification services (e.g. Smart Dollar Auth, 3DS).

    Attributes:
        method (CardVerificationMethod): The method used for card
            verification.

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "method": 'method'
    }

    _optionals = [
        'method',
    ]

    def __init__(self,
                 method='SCA_WHEN_REQUIRED'):
        """Constructor for the CardVerification class"""

        # Initialize members of the class
        self.method = method 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        method = dictionary.get("method") if dictionary.get("method") else 'SCA_WHEN_REQUIRED'
        # Return an object of this model
        return cls(method)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class RefundStatus(object):

    """Implementation of the 'Refund Status' enum.

    The status of the refund.

    Attributes:
        CANCELLED: TODO: type description here.
        FAILED: TODO: type description here.
        PENDING: TODO: type description here.
        COMPLETED: TODO: type description here.

    """
    CANCELLED = 'CANCELLED'

    FAILED = 'FAILED'

    PENDING = 'PENDING'

    COMPLETED = 'COMPLETED'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class CaptureIncompleteReason(object):

    """Implementation of the 'Capture Incomplete Reason' enum.

    The reason why the captured payment status is `PENDING` or `DENIED`.

    Attributes:
        BUYER_COMPLAINT: TODO: type description here.
        CHARGEBACK: TODO: type description here.
        ECHECK: TODO: type description here.
        INTERNATIONAL_WITHDRAWAL: TODO: type description here.
        OTHER: TODO: type description here.
        PENDING_REVIEW: TODO: type description here.
        RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION: TODO: type description
            here.
        REFUNDED: TODO: type description here.
        TRANSACTION_APPROVED_AWAITING_FUNDING: TODO: type description here.
        UNILATERAL: TODO: type description here.
        VERIFICATION_REQUIRED: TODO: type description here.
        DECLINED_BY_RISK_FRAUD_FILTERS: TODO: type description here.

    """
    BUYER_COMPLAINT = 'BUYER_COMPLAINT'

    CHARGEBACK = 'CHARGEBACK'

    ECHECK = 'ECHECK'

    INTERNATIONAL_WITHDRAWAL = 'INTERNATIONAL_WITHDRAWAL'

    OTHER = 'OTHER'

    PENDING_REVIEW = 'PENDING_REVIEW'

    RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION = 'RECEIVING_PREFERENCE_MANDATES_MANUAL_ACTION'

    REFUNDED = 'REFUNDED'

    TRANSACTION_APPROVED_AWAITING_FUNDING = 'TRANSACTION_APPROVED_AWAITING_FUNDING'

    UNILATERAL = 'UNILATERAL'

    VERIFICATION_REQUIRED = 'VERIFICATION_REQUIRED'

    DECLINED_BY_RISK_FRAUD_FILTERS = 'DECLINED_BY_RISK_FRAUD_FILTERS'




# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""
from paypalserversdk.api_helper import APIHelper
from paypalserversdk.models.link_description import LinkDescription
from paypalserversdk.models.money import Money
from paypalserversdk.models.payee_base import PayeeBase
from paypalserversdk.models.refund_status_details import RefundStatusDetails
from paypalserversdk.models.seller_payable_breakdown import SellerPayableBreakdown


class Refund(object):

    """Implementation of the 'Refund' model.

    The refund information.

    Attributes:
        status (RefundStatus): The status of the refund.
        status_details (RefundStatusDetails): The details of the refund
            status.
        id (str): The PayPal-generated ID for the refund.
        amount (Money): The currency and amount for a financial transaction,
            such as a balance or payment due.
        invoice_id (str): The API caller-provided external invoice number for
            this order. Appears in both the payer's transaction history and
            the emails that the payer receives.
        custom_id (str): The API caller-provided external ID. Used to
            reconcile API caller-initiated transactions with PayPal
            transactions. Appears in transaction and settlement reports.
        acquirer_reference_number (str): Reference ID issued for the card
            transaction. This ID can be used to track the transaction across
            processors, card brands and issuing banks.
        note_to_payer (str): The reason for the refund. Appears in both the
            payer's transaction history and the emails that the payer
            receives.
        seller_payable_breakdown (SellerPayableBreakdown): The breakdown of
            the refund.
        payer (PayeeBase): The details for the merchant who receives the funds
            and fulfills the order. The merchant is also known as the payee.
        links (List[LinkDescription]): An array of related [HATEOAS
            links](/docs/api/reference/api-responses/#hateoas-links).
        create_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>
        update_time (str): The date and time, in [Internet date and time
            format](https://tools.ietf.org/html/rfc3339#section-5.6). Seconds
            are required while fractional seconds are
            optional.<blockquote><strong>Note:</strong> The regular expression
            provides guidance but does not reject all invalid
            dates.</blockquote>

    """

    # Create a mapping from Model property names to API property names
    _names = {
        "status": 'status',
        "status_details": 'status_details',
        "id": 'id',
        "amount": 'amount',
        "invoice_id": 'invoice_id',
        "custom_id": 'custom_id',
        "acquirer_reference_number": 'acquirer_reference_number',
        "note_to_payer": 'note_to_payer',
        "seller_payable_breakdown": 'seller_payable_breakdown',
        "payer": 'payer',
        "links": 'links',
        "create_time": 'create_time',
        "update_time": 'update_time'
    }

    _optionals = [
        'status',
        'status_details',
        'id',
        'amount',
        'invoice_id',
        'custom_id',
        'acquirer_reference_number',
        'note_to_payer',
        'seller_payable_breakdown',
        'payer',
        'links',
        'create_time',
        'update_time',
    ]

    def __init__(self,
                 status=APIHelper.SKIP,
                 status_details=APIHelper.SKIP,
                 id=APIHelper.SKIP,
                 amount=APIHelper.SKIP,
                 invoice_id=APIHelper.SKIP,
                 custom_id=APIHelper.SKIP,
                 acquirer_reference_number=APIHelper.SKIP,
                 note_to_payer=APIHelper.SKIP,
                 seller_payable_breakdown=APIHelper.SKIP,
                 payer=APIHelper.SKIP,
                 links=APIHelper.SKIP,
                 create_time=APIHelper.SKIP,
                 update_time=APIHelper.SKIP):
        """Constructor for the Refund class"""

        # Initialize members of the class
        if status is not APIHelper.SKIP:
            self.status = status 
        if status_details is not APIHelper.SKIP:
            self.status_details = status_details 
        if id is not APIHelper.SKIP:
            self.id = id 
        if amount is not APIHelper.SKIP:
            self.amount = amount 
        if invoice_id is not APIHelper.SKIP:
            self.invoice_id = invoice_id 
        if custom_id is not APIHelper.SKIP:
            self.custom_id = custom_id 
        if acquirer_reference_number is not APIHelper.SKIP:
            self.acquirer_reference_number = acquirer_reference_number 
        if note_to_payer is not APIHelper.SKIP:
            self.note_to_payer = note_to_payer 
        if seller_payable_breakdown is not APIHelper.SKIP:
            self.seller_payable_breakdown = seller_payable_breakdown 
        if payer is not APIHelper.SKIP:
            self.payer = payer 
        if links is not APIHelper.SKIP:
            self.links = links 
        if create_time is not APIHelper.SKIP:
            self.create_time = create_time 
        if update_time is not APIHelper.SKIP:
            self.update_time = update_time 

    @classmethod
    def from_dictionary(cls,
                        dictionary):
        """Creates an instance of this model from a dictionary

        Args:
            dictionary (dictionary): A dictionary representation of the object
            as obtained from the deserialization of the server's response. The
            keys MUST match property names in the API description.

        Returns:
            object: An instance of this structure class.

        """

        if dictionary is None:
            return None

        # Extract variables from the dictionary
        status = dictionary.get("status") if dictionary.get("status") else APIHelper.SKIP
        status_details = RefundStatusDetails.from_dictionary(dictionary.get('status_details')) if 'status_details' in dictionary.keys() else APIHelper.SKIP
        id = dictionary.get("id") if dictionary.get("id") else APIHelper.SKIP
        amount = Money.from_dictionary(dictionary.get('amount')) if 'amount' in dictionary.keys() else APIHelper.SKIP
        invoice_id = dictionary.get("invoice_id") if dictionary.get("invoice_id") else APIHelper.SKIP
        custom_id = dictionary.get("custom_id") if dictionary.get("custom_id") else APIHelper.SKIP
        acquirer_reference_number = dictionary.get("acquirer_reference_number") if dictionary.get("acquirer_reference_number") else APIHelper.SKIP
        note_to_payer = dictionary.get("note_to_payer") if dictionary.get("note_to_payer") else APIHelper.SKIP
        seller_payable_breakdown = SellerPayableBreakdown.from_dictionary(dictionary.get('seller_payable_breakdown')) if 'seller_payable_breakdown' in dictionary.keys() else APIHelper.SKIP
        payer = PayeeBase.from_dictionary(dictionary.get('payer')) if 'payer' in dictionary.keys() else APIHelper.SKIP
        links = None
        if dictionary.get('links') is not None:
            links = [LinkDescription.from_dictionary(x) for x in dictionary.get('links')]
        else:
            links = APIHelper.SKIP
        create_time = dictionary.get("create_time") if dictionary.get("create_time") else APIHelper.SKIP
        update_time = dictionary.get("update_time") if dictionary.get("update_time") else APIHelper.SKIP
        # Return an object of this model
        return cls(status,
                   status_details,
                   id,
                   amount,
                   invoice_id,
                   custom_id,
                   acquirer_reference_number,
                   note_to_payer,
                   seller_payable_breakdown,
                   payer,
                   links,
                   create_time,
                   update_time)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""


class StoreInVaultInstruction(object):

    """Implementation of the 'Store In Vault Instruction' enum.

    Defines how and when the payment source gets vaulted.

    Attributes:
        ON_SUCCESS: TODO: type description here.

    """
    ON_SUCCESS = 'ON_SUCCESS'




from apimatic_core.http.response.api_response import ApiResponse
from apimatic_core.http.response.http_response import HttpResponse


class ApiResponse(ApiResponse):

    """Http response received.

    Attributes:
        status_code (int): The status code response from the server that
            corresponds to this response.
        reason_phrase (string): The reason phrase returned by the server.
        headers (dict): A dictionary of headers (key : value) that were
            returned with the response
        text (string): The Raw body of the HTTP Response as a string
        request (HttpRequest): The request that resulted in this response.
        body (Object): The data field specified for the response
        errors (Array<String>): Any errors returned by the server

    """

    def __init__(self, http_response,
                 body=None,
                 errors=None):

        """The Constructor

        Args:
            http_response (HttpResponse): The original, raw response from the api
            body (Object): The body of the actual response
            errors (Array<String>): Any errors returned by the server

        """

        super().__init__(http_response, body, errors)

    def __repr__(self):
        return '<ApiResponse {}>'.format(self.text)

    @classmethod
    def create(cls, parent_instance):
        return cls(HttpResponse(parent_instance.status_code, parent_instance.reason_phrase,
                                parent_instance.headers, parent_instance.text, parent_instance.request),
                   parent_instance.body, parent_instance.errors)



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.payment_authorization import PaymentAuthorization
from paypalserversdk.models.captured_payment import CapturedPayment
from paypalserversdk.models.refund import Refund
from paypalserversdk.exceptions.error_exception import ErrorException
from paypalserversdk.exceptions.api_exception import APIException


class PaymentsController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(PaymentsController, self).__init__(config)

    def authorizations_get(self,
                           authorization_id):
        """Does a GET request to /v2/payments/authorizations/{authorization_id}.

        Shows details for an authorized payment, by ID.

        Args:
            authorization_id (str): The ID of the authorized payment for which
                to show details.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>200 OK</code> status
                code and a JSON response body that shows authorization
                details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(authorization_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def authorizations_capture(self,
                               options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/capture.

        Captures an authorized payment, by ID.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to capture.
                    pay_pal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    body -- CaptureRequest -- TODO: type description here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>201 Created</code>
                status code and a JSON response body that shows captured
                payment details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/capture')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CapturedPayment.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The server has detected a conflict while processing this request.', ErrorException)
            .local_error('422', 'The request failed because it is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def authorizations_reauthorize(self,
                                   options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/reauthorize.

        Reauthorizes an authorized PayPal account payment, by ID. To ensure
        that funds are still available, reauthorize a payment after its
        initial three-day honor period expires. Within the 29-day
        authorization period, you can issue multiple re-authorizations after
        the honor period expires.<br/><br/>If 30 days have transpired since
        the date of the original authorization, you must create an authorized
        payment instead of reauthorizing the original authorized
        payment.<br/><br/>A reauthorized payment itself has a new honor period
        of three days.<br/><br/>You can reauthorize an authorized payment from
        4 to 29 days after the 3-day honor period. The allowed amount depends
        on context and geography, for example in US it is up to 115% of the
        original authorized amount, not to exceed an increase of $75
        USD.<br/><br/>Supports only the `amount` request
        parameter.<blockquote><strong>Note:</strong> This request is currently
        not supported for Partner use cases.</blockquote>

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to reauthorize.
                    pay_pal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    body -- ReauthorizeRequest -- TODO: type description
                        here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>201 Created</code>
                status code and a JSON response body that shows the
                reauthorized payment details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/reauthorize')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def authorizations_void(self,
                            options=dict()):
        """Does a POST request to /v2/payments/authorizations/{authorization_id}/void.

        Voids, or cancels, an authorized payment, by ID. You cannot void an
        authorized payment that has been fully captured.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    authorization_id -- str -- The PayPal-generated ID for the
                        authorized payment to void.
                    pay_pal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion).<blockquote><strong>Note:</st
                        rong>For three party transactions in which a partner
                        is managing the API calls on behalf of a merchant, the
                        partner must identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.</blockquote>
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>200 OK</code> status
                code and a JSON response body that shows authorization
                details. This response is returned when the Prefer header is
                set to return=representation.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/authorizations/{authorization_id}/void')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('authorization_id')
                            .value(options.get('authorization_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('pay_pal_auth_assertion', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentAuthorization.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The request failed because a previous call for the given resource is in progress.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def captures_get(self,
                     capture_id):
        """Does a GET request to /v2/payments/captures/{capture_id}.

        Shows details for a captured payment, by ID.

        Args:
            capture_id (str): The PayPal-generated ID for the captured payment
                for which to show details.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>200 OK</code> status
                code and a JSON response body that shows captured payment
                details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/captures/{capture_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('capture_id')
                            .value(capture_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CapturedPayment.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def captures_refund(self,
                        options=dict()):
        """Does a POST request to /v2/payments/captures/{capture_id}/refund.

        Refunds a captured payment, by ID. For a full refund, include an empty
        payload in the JSON request body. For a partial refund, include an
        <code>amount</code> object in the JSON request body.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    capture_id -- str -- The PayPal-generated ID for the
                        captured payment to refund.
                    pay_pal_request_id -- str -- The server stores keys for 45
                        days.
                    prefer -- str -- The preferred server response upon
                        successful completion of the request. Value
                        is:<ul><li><code>return=minimal</code>. The server
                        returns a minimal response to optimize communication
                        between the API caller and the server. A minimal
                        response includes the <code>id</code>,
                        <code>status</code> and HATEOAS
                        links.</li><li><code>return=representation</code>. The
                        server returns a complete resource representation,
                        including the current state of the
                        resource.</li></ul>
                    pay_pal_auth_assertion -- str -- An API-caller-provided
                        JSON Web Token (JWT) assertion that identifies the
                        merchant. For details, see
                        [PayPal-Auth-Assertion](/docs/api/reference/api-request
                        s/#paypal-auth-assertion).<blockquote><strong>Note:</st
                        rong>For three party transactions in which a partner
                        is managing the API calls on behalf of a merchant, the
                        partner must identify the merchant using either a
                        PayPal-Auth-Assertion header or an access token with
                        target_subject.</blockquote>
                    body -- RefundRequest -- TODO: type description here.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>201 Created</code>
                status code and a JSON response body that shows refund
                details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/captures/{capture_id}/refund')
            .http_method(HttpMethodEnum.POST)
            .template_param(Parameter()
                            .key('capture_id')
                            .value(options.get('capture_id', None))
                            .should_encode(True))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Prefer')
                          .value(options.get('prefer', None)))
            .header_param(Parameter()
                          .key('PayPal-Auth-Assertion')
                          .value(options.get('pay_pal_auth_assertion', None)))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Refund.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'The request failed because it is not well-formed or is syntactically incorrect or violates schema.', ErrorException)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('409', 'The request failed because a previous call for the given resource is in progress.', ErrorException)
            .local_error('422', 'The request failed because it either is semantically incorrect or failed business validation.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()

    def refunds_get(self,
                    refund_id):
        """Does a GET request to /v2/payments/refunds/{refund_id}.

        Shows details for a refund, by ID.

        Args:
            refund_id (str): The PayPal-generated ID for the refund for which
                to show details.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. A
                successful request returns the HTTP <code>200 OK</code> status
                code and a JSON response body that shows refund details.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v2/payments/refunds/{refund_id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('refund_id')
                            .value(refund_id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(Refund.from_dictionary)
            .is_api_response(True)
            .local_error('401', 'Authentication failed due to missing authorization header, or invalid authentication credentials.', ErrorException)
            .local_error('403', 'The request failed because the caller has insufficient permissions.', ErrorException)
            .local_error('404', 'The request failed because the resource does not exist.', ErrorException)
            .local_error('500', 'The request failed because an internal server error occurred.', APIException)
            .local_error('default', 'The error response.', ErrorException)
        ).execute()



# -*- coding: utf-8 -*-

"""
paypalserversdk

This file was automatically generated by APIMATIC v3.0 (
 https://www.apimatic.io ).
"""

from paypalserversdk.api_helper import APIHelper
from paypalserversdk.configuration import Server
from paypalserversdk.http.api_response import ApiResponse
from paypalserversdk.controllers.base_controller import BaseController
from apimatic_core.request_builder import RequestBuilder
from apimatic_core.response_handler import ResponseHandler
from apimatic_core.types.parameter import Parameter
from paypalserversdk.http.http_method_enum import HttpMethodEnum
from apimatic_core.authentication.multiple.single_auth import Single
from paypalserversdk.models.payment_token_response import PaymentTokenResponse
from paypalserversdk.models.customer_vault_payment_tokens_response import CustomerVaultPaymentTokensResponse
from paypalserversdk.models.setup_token_response import SetupTokenResponse
from paypalserversdk.exceptions.error_exception import ErrorException


class VaultController(BaseController):

    """A Controller to access Endpoints in the paypalserversdk API."""
    def __init__(self, config):
        super(VaultController, self).__init__(config)

    def payment_tokens_create(self,
                              options=dict()):
        """Does a POST request to /v3/vault/payment-tokens.

        Creates a Payment Token from the given payment source and adds it to
        the Vault of the associated customer.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    pay_pal_request_id -- str -- The server stores keys for 3
                        hours.
                    body -- PaymentTokenRequest -- Payment Token creation with
                        a financial instrument and an optional customer_id.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers.
                Idempotent response for a successful creation of payment
                token.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/payment-tokens')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentTokenResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'Request contains reference to resources that do not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()

    def customer_payment_tokens_get(self,
                                    options=dict()):
        """Does a GET request to /v3/vault/payment-tokens.

        Returns all payment tokens for a customer.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    customer_id -- str -- A unique identifier representing a
                        specific customer in merchant's/partner's system or
                        records.
                    page_size -- int -- A non-negative, non-zero integer
                        indicating the maximum number of results to return at
                        one time.
                    page -- int -- A non-negative, non-zero integer
                        representing the page of the results.
                    total_required -- bool -- A boolean indicating total
                        number of items (total_items) and pages (total_pages)
                        are expected to be returned in the response.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers.
                Successful execution.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/payment-tokens')
            .http_method(HttpMethodEnum.GET)
            .query_param(Parameter()
                         .key('customer_id')
                         .value(options.get('customer_id', None)))
            .query_param(Parameter()
                         .key('page_size')
                         .value(options.get('page_size', None)))
            .query_param(Parameter()
                         .key('page')
                         .value(options.get('page', None)))
            .query_param(Parameter()
                         .key('total_required')
                         .value(options.get('total_required', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(CustomerVaultPaymentTokensResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()

    def payment_tokens_get(self,
                           id):
        """Does a GET request to /v3/vault/payment-tokens/{id}.

        Returns a readable representation of vaulted payment source associated
        with the payment token id.

        Args:
            id (str): ID of the payment token.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers.
                Successful execution.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/payment-tokens/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(PaymentTokenResponse.from_dictionary)
            .is_api_response(True)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()

    def payment_tokens_delete(self,
                              id):
        """Does a DELETE request to /v3/vault/payment-tokens/{id}.

        Delete the payment token associated with the payment token id.

        Args:
            id (str): ID of the payment token.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. The
                server has successfully executed the method, but there is no
                entity body to return.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/payment-tokens/{id}')
            .http_method(HttpMethodEnum.DELETE)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()

    def setup_tokens_create(self,
                            options=dict()):
        """Does a POST request to /v3/vault/setup-tokens.

        Creates a Setup Token from the given payment source and adds it to the
        Vault of the associated customer.

        Args:
            options (dict, optional): Key-value pairs for any of the
                parameters to this API Endpoint. All parameters to the
                endpoint are supplied through the dictionary with their names
                being the key and their desired values being the value. A list
                of parameters that can be used are::

                    pay_pal_request_id -- str -- The server stores keys for 3
                        hours.
                    body -- SetupTokenRequest -- Setup Token creation with a
                        instrument type optional financial instrument details
                        and customer_id.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers.
                Idempotent response for a successful creation of setup token.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/setup-tokens')
            .http_method(HttpMethodEnum.POST)
            .header_param(Parameter()
                          .key('PayPal-Request-Id')
                          .value(options.get('pay_pal_request_id', None)))
            .header_param(Parameter()
                          .key('Content-Type')
                          .value('application/json'))
            .body_param(Parameter()
                        .value(options.get('body', None)))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .body_serializer(APIHelper.json_serialize)
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SetupTokenResponse.from_dictionary)
            .is_api_response(True)
            .local_error('400', 'Request is not well-formed, syntactically incorrect, or violates schema.', ErrorException)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()

    def setup_tokens_get(self,
                         id):
        """Does a GET request to /v3/vault/setup-tokens/{id}.

        Returns a readable representation of temporarily vaulted payment
        source associated with the setup token id.

        Args:
            id (str): ID of the setup token.

        Returns:
            ApiResponse: An object with the response value as well as other
                useful information such as status codes and headers. Found
                requested setup-token, returned a payment method associated
                with the token.

        Raises:
            APIException: When an error occurs while fetching the data from
                the remote API. This exception includes the HTTP Response
                code, an error message, and the HTTP body that was received in
                the request.

        """

        return super().new_api_call_builder.request(
            RequestBuilder().server(Server.DEFAULT)
            .path('/v3/vault/setup-tokens/{id}')
            .http_method(HttpMethodEnum.GET)
            .template_param(Parameter()
                            .key('id')
                            .value(id)
                            .should_encode(True))
            .header_param(Parameter()
                          .key('accept')
                          .value('application/json'))
            .auth(Single('Oauth2'))
        ).response(
            ResponseHandler()
            .deserializer(APIHelper.json_deserialize)
            .deserialize_into(SetupTokenResponse.from_dictionary)
            .is_api_response(True)
            .local_error('403', 'Authorization failed due to insufficient permissions.', ErrorException)
            .local_error('404', 'The specified resource does not exist.', ErrorException)
            .local_error('422', 'The requested action could not be performed, semantically incorrect, or failed business validation.', ErrorException)
            .local_error('500', 'An internal server error has occurred.', ErrorException)
        ).execute()
